iProblem2.InitConstants = function() {
	START_SUN_ANGLE  = 1.87;	// SunAngle isin [PI/2,PI]
	FINISH_SUN_ANGLE = 2.11; 	// 
	SUN_RADIUS = 10000000;
	SUN_SEASON_ANGLE = Math.PI*(1/3); 	// Season angle isin [0,PI/2]
	
	START_STICK_HEIGHT  = 1.2;
	FINISH_STICK_HEIGHT = 1.7;
	
	START_TREE_HEIGHT  = 7;   // 7
	FINISH_TREE_HEIGHT = 10;  // 10

	DIST_FROM_SCREEN_TO_USER = 1000;
	Y_POINT_OF_ORIGIN = 700;
	HORIZONT = 100;
	METER_TO_PIXEL_COEF = 200;
};

iProblem2.InitData = function() {
	SunAngle = RandomInLims(START_SUN_ANGLE, FINISH_SUN_ANGLE);
	StickHeight = RoundDeg(RandomInLims(START_STICK_HEIGHT, FINISH_STICK_HEIGHT),2);
	TreeHeight = RoundDeg(RandomInLims(START_TREE_HEIGHT, FINISH_TREE_HEIGHT),1);
	ShowForestShadows();
};

iProblem2.StickShadowLength = function() {return Math.abs(StickHeight * Math.tan(AngleToSun()));};
iProblem2.TreeShadowLength = function() {return Math.abs(TreeHeight * Math.tan(AngleToSun()));};
iProblem2.TreeToStickCoef = function() {return TreeHeight / StickHeight;};
iProblem2.AngleToSun = function() {
	return Math.acos(Math.sin(SUN_SEASON_ANGLE)*Math.sin(SunAngle));	
};

// Converts radial coordinates to rectangle
// x is -> oriented
// y is v oriented
// z is oriented into the screen
// (0,0,0) is the center of the screen
iProblem2.GetRectCoord = function(dist, hourAng, seasonAng) {
	return new Array(
		dist*Math.sin(seasonAng)*Math.cos(hourAng),
		-dist*Math.sin(seasonAng)*Math.sin(hourAng),
		dist*Math.cos(seasonAng)
	);
};

iProblem2.GetRectToScreenCoef = function(z) {
	return 1/(1 + z / DIST_FROM_SCREEN_TO_USER);
}

// returns projected coordinates
iProblem2.GetScreenCoord = function(x,y,z) {
	var coef = GetRectToScreenCoef(z);
	return new Array(x*coef,(y+Y_POINT_OF_ORIGIN+HORIZONT)*coef - HORIZONT);
};

iProblem2.GetSunCoord = function() {
	return GetRectCoord(SUN_RADIUS, SunAngle, SUN_SEASON_ANGLE);
};

// x_,y_ position on screen
// position in 3D space
// y in 3D is considered to be 0
iProblem2.ConvScreenToRectCoord = function(x_,y_) {
	var coef = (Y_POINT_OF_ORIGIN + HORIZONT)/(y_ + HORIZONT);
	var result =  new Array(
		x_*coef,
		0,
		DIST_FROM_SCREEN_TO_USER * (coef - 1)
	);
	return result;
};

// Caclulates point on plane of the end of the shadow
// input - 3D coordinates of the top of the object
iProblem2.GetShadowEndCoord = function(x,y,z) {
	// x = 0; =>
	// ray function F(t) = S + (H - S)t
	// S - sun coordinates
	// H - 3D coordinates of the top of the object
	var S = GetSunCoord();
    var t = S[1]/(S[1]-y);
    var result = new Array(
    	S[0] * (1-t) + x*t,
    	0,
    	S[2] * (1-t) + z*t
    );
    return result;
};

iProblem2.InsertPoint = function (x, y, color, clear) {
	if (iTest == undefined) {
		createEmptyMovieClip("iTest",2);
		iTest._x = 0;
		iTest._y = 0;
	}
	if (clear) iTest.clear();
	iTest.lineStyle(1,color,100);
	iTest.moveTo(-600,y);
	iTest.lineTo(+600,y);
	iTest.moveTo(x,-600);
	iTest.lineTo(x,+600);
};

iProblem2.BuildShadow = function (iPict, iShadow, height) {
	var x_ = iPict._x+iPict._parent._x;
	var y_ = iPict._y+iPict._parent._y;
	height *= METER_TO_PIXEL_COEF;
	var top3D  = ConvScreenToRectCoord(x_,y_,0);
	top3D[1] -= height;
	var shEnd3D = GetShadowEndCoord(top3D[0],top3D[1],top3D[2]);
	var shEndScr = GetScreenCoord(shEnd3D[0],shEnd3D[1],shEnd3D[2]);

//	InsertPoint(shEndScr[0],shEndScr[1],0x000000,true);

	var dx = shEndScr[0] - x_;
	var dy = shEndScr[1] - y_;
	height = Math.sqrt(dx*dx+dy*dy);
	var ang = Math.atan2(dy,dx);
//	trace(height + ", " + ang);
	iShadow._rotation = 0;
	iShadow._height = height;// * 100 / iShadow._height;
	iShadow._rotation = ang*180/Math.PI - 90;
};

iProblem2.GetScale = function (iPict, mHeight, pHeight) {
	var x_ = iPict._x+iPict._parent._x;
	var y_ = iPict._y+iPict._parent._y;
	if (pHeight == undefined)
		pHeight = iPict._height * 100 / iPict._yscale;
	var height = mHeight * METER_TO_PIXEL_COEF / pHeight;
	var top3D  = ConvScreenToRectCoord(x_,y_,0);
	var coef = GetRectToScreenCoef(top3D[2])*100;	
	return height*coef;
}
	
iProblem2.RandomInLims = function(startValue,finishValue) {
	res = startValue + Math.random()*(finishValue - startValue);
	return res;
};

iProblem2.RoundDeg = function(number,deg) {
	return Math.round(number * Math.pow(10,deg)) / Math.pow(10,deg);
};

iProblem2.CheckData = function(number, etalon, deg) {
	return Math.abs(number - etalon) < Math.pow(10,-deg)/2;
};